#include "treeview.h"

#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>
#include <TCutG.h>
#include <TString.h>
#include <TH1.h>

#include <iostream>


TreeViewer::TreeViewer(QWidget *) : file_(0), tree_(0), cutList_(),tmpCut_(0)
{
  setupUi(this);
  cmbCutOpt->addItem("add","||");
  cmbCutOpt->addItem("remove","remove");
  cmbCutOpt->addItem("intersect","&&");

}

TreeViewer::~TreeViewer()
{
  if(file_) file_->Close();
  if(tmpCut_) tmpCut_->Delete();
  cutList_.clear();
}


bool TreeViewer::openFile(TString fileName) 
{
  file_ = TFile::Open(fileName);
  if(file_) { 
    std::cout << "Successfully opened file " << fileName << std::endl;
    return true; 
  } else {
    // no message since TFile::Open will issue a warning upon failure
    return false;
  }
}

bool TreeViewer::getTree(TString treeName)
{
  if(!file_) {
    std::cerr << "no valid file was opened" << std::endl;
    return false;
  }
  tree_ = static_cast<TTree*>(file_->Get(treeName));
  if(!tree_) {
    std::cerr << "Could not get TTree " << treeName << " from file " << file_->GetName();
    return false;
  } else {
    std::cout << "TTree " << treeName << " was retrieved from file " << file_->GetName() << std::endl;
  }
  return true;
}


void TreeViewer::on_btnCut_clicked()
{
  if(tmpCut_) {
    tmpCut_->SetEditable(kFALSE);
    delete tmpCut_;
    tmpCut_ = 0;
    btnCut->setText("New Cut");
    this->update();
    return;
  }
  TH1* hist = static_cast<TH1*>(gROOT->FindObject("htemp"));
  if(!hist) return;
  double x[2]  = {this->getCanvas()->GetUxmin(),this->getCanvas()->GetUxmax()};
  double y[2]  = {this->getCanvas()->GetUymin(),this->getCanvas()->GetUymax()};
  double xdist = x[1] - x[0];
  double ydist = y[1] - y[0];
  x[0] += 0.1*xdist; x[1] -= 0.1*xdist;
  y[0] += 0.1*ydist; y[1] -= 0.1*ydist;
  double *cutX = new double[5];
  cutX[0] = x[0]; cutX[1] = x[0]; cutX[2] = x[1]; cutX[3] = x[1]; cutX[4] = x[0];
  double *cutY = new double[5];
  cutY[0] = y[0]; cutY[1] = y[1]; cutY[2] = y[1]; cutY[3] = y[0]; cutY[4] = y[0];
  tmpCut_ = new TCutG(Form("cut%d",cutList_.count()),5,cutX,cutY);
  tmpCut_->SetVarX("py");
  tmpCut_->SetVarY("px");
  tmpCut_->SetLineColor(kRed);
  tmpCut_->Draw("L*same");
  btnCut->setText("Remove Cut");

  

  this->update();

}

void TreeViewer::on_btnSelect_clicked()
{
  if(tmpCut_) {
    QString cutOpt = cmbCutOpt->itemData(cmbCutOpt->currentIndex()).toString();
    if( cutList_.count() == 0 ) {
      cutOpt = "";
    }
    QString cutName(tmpCut_->GetName());
    cutName.prepend("list_");
    TCutG* cutClone = static_cast<TCutG*>(tmpCut_->Clone(cutName.toStdString().c_str()));
    cutList_.append(QPair<QString,TCutG*>(cutOpt,cutClone));
    std::cout << "Appended cut" << std::endl;
    delete tmpCut_;
    tmpCut_ = 0;
    btnCut->setText("New Cut");
    this->draw();

  }
}

void TreeViewer::draw()
{
  if(!tree_) {
    std::cerr << "No tree was loaded.\nPlease place the hsimple.root file from the corresponding root tutorial in this directory\n"
	      << "or change the program accordingly to use a custom file" << std::endl;
    exit(EXIT_FAILURE);
  }
  QString cutString = QString("");
  tree_->SetMarkerStyle(7);
  tree_->SetLineColor(kBlack);
  tree_->SetMarkerColor(kBlack);
  tree_->Draw("px:py",cutString.toStdString().c_str());
  if(cutList_.count() > 0 ) {
    QString TCutString = "";
    for(int i = 0; i < cutList_.count(); ++i ) {
      if( cutList_.at(i).first == "remove" ) {
	TCutString.prepend("((");
	TCutString += ")*(!"+QString(cutList_.at(i).second->GetName())+"))";
      } else {
	TCutString += cutList_.at(i).first+QString(cutList_.at(i).second->GetName());
      }
    }
    cutString += TCutString;
    tree_->SetLineColor(kRed);
    tree_->SetMarkerColor(kRed);
    tree_->Draw("px:py",cutString.toStdString().c_str(),"same");
    std::cout << cutString.toStdString() << std::endl;
  }

  this->update();
}


void TreeViewer::update()
{
  qCanvas->Refresh();
}
